// ***************************************************************************************
// File: thread.cpp
// Description: session threads
// Author: Pierre MARC
// Copyright (C) 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Nirva Systems
//
// This file is part of Nirva Application Platform.
// 
// Nirva Application Platform is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.

// Nirva Application Platform is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public License
// along with Nirva Application Platform.  If not, see <http://www.gnu.org/licenses/>.
//
// ***************************************************************************************
// Comments: 
// ***************************************************************************************


#include "nvs.h"

#ifdef NV_SERVER
	#ifdef WIN32
		#ifdef _DEBUG
			#include <stdlib.h>
			#include <crtdbg.h>
			#define NVNEW new( _NORMAL_BLOCK, __FILE__, __LINE__)
		#else // _DEBUG
			#define NVNEW new
		#endif // _DEBUG
	#else	//WIN32
		#define NVNEW new
	#endif	//WIN32
#else	//NV_SERVER
	#define NVNEW new
#endif	//NV_SERVER

using namespace std;


NvSessionThread::NvSessionThread()
{
	b_Initialized = false;
	m_sSessionId = "";
	m_sCallingSessionId = "";
	m_sProcName = "";
	m_sProcNameFailed = "";
}



// Client thread main function
int NvSessionThread::Main()
{
	if(m_sSessionId.length() == 0)
		return 0;
	NvSession *Session = GetMain()->Sessions.GetSession(m_sSessionId.c_str());
	if(Session == NULL)
		return 0;
	
	if(m_sProcName.length())
	{
		// Do the main loop here
		// We prepare a command
		NvCommand Command;
		Command.m_iCommandSource = NV_SOURCE_SERVER;
		Command.m_sService = "SYSTEM";
		Command.m_sClass = "MISC";
		Command.m_sCommand = "NOP";
		Command.m_sInContainer = NV_CONTAINER_DEFAULT_NAME;
		Command.m_sOutContainer = NV_CONTAINER_DEFAULT_NAME;
		Command.Parameters.SetValue("NV_PROC", m_sProcName);
		Command.Parameters.SetValue("NV_CALLING_SESSION", m_sCallingSessionId);


		// Run the procedure here
		bool Result = true;
		try
		{
			Result = Session->Command(&Command);
		}
		catch(...)
		{
			Result = false;
		}
			
		// Call the error procedure if one has been defined
		if(!Result && m_sProcNameFailed.length())
		{
			NvCommand ErrorCommand;
			ErrorCommand.m_iCommandSource = NV_SOURCE_SERVER;
			ErrorCommand.m_sService = "SYSTEM";
			ErrorCommand.m_sClass = "MISC";
			ErrorCommand.m_sCommand = "NOP";
			ErrorCommand.m_sInContainer = NV_CONTAINER_DEFAULT_NAME;
			ErrorCommand.m_sOutContainer = NV_CONTAINER_DEFAULT_NAME;
			ErrorCommand.Parameters.SetValue("NV_PROC", m_sProcNameFailed);
			char Buf[20];
			sprintf(Buf, "%d", Command.Error.m_iErrorCode);
			ErrorCommand.Parameters.SetValue("NV_ERROR_CODE", Buf);
			ErrorCommand.Parameters.SetValue("NV_ERROR_SERVICE", Command.Error.m_sErrorService.c_str());
			ErrorCommand.Parameters.SetValue("NV_ERROR_CLASS", Command.Error.m_sErrorClass.c_str());
			ErrorCommand.Parameters.SetValue("NV_ERROR_DESC", Command.Error.GetErrorDesc(Session->m_sLanguage.c_str()).c_str());
			ErrorCommand.Parameters.SetValue("NV_ERROR_INFO", Command.Error.m_sErrorInfo.c_str());
			try
			{
				Session->Command(&ErrorCommand);
			}
			catch(...)
			{
			}
		}
	}

	// Free the session
	GetMain()->Sessions.FreeSession(Session->m_sIdent.c_str());
	return 0;
}


// called when an exception occurs (see class NvTask)
void NvSessionThread::OnException(string sInfo)
{
//	throw this;
}



// called before the Main function (see class NvTask)
bool NvSessionThread::OnInit()
{
	NvTask::OnInit();	// call the base class

	if(!m_bSameThread)
		GetMain()->Threads.Add(THREAD_TYPE_SESSION);
	
	char sThreadId[50];
	sprintf(sThreadId, "%lu", NvGetCurrentThreadId());
	if(!m_bSameThread)
		GetMain()->Logs.Write("Threads", "Init thread", "Type SESSION", NV_LOG_INFO, 1, sThreadId, "system");

	// Initialize the random generator for this thread
#ifdef WIN32
	int ThreadHandle = GetCurrentThreadId();						// Thread handle for window
#else
	pthread_t ThreadHandle = pthread_self();					// Thread handle for unix
#endif

	unsigned int randinit = (unsigned int)ThreadHandle;
	randinit *= (unsigned int)time(NULL);
	srand(randinit);

	b_Initialized = true;
	
	if(!m_bSameThread)
		GetMain()->Logs.Write("Threads", "Thread started", "Type SESSION", NV_LOG_INFO, 1, sThreadId, "system");

	return true;
}



// Start the session thread
string NvSessionThread::Start(const char *CallingSessionId, const char *AppName, const char *ProcName, const char *ProcNameFailed, const char *UserName, const char *OpenProc, const char *CloseProc, const char *Language)
{
	m_sSessionId = "";
	if(GetMain()->CheckOverTime())
		return m_sSessionId;
	NvSession *Session = GetMain()->Sessions.GetSession(NULL);
	if(Session == NULL)
		return m_sSessionId;

	m_sCallingSessionId = CallingSessionId;

	string AppliNameHere = AppName;
	CleanString(AppliNameHere);
	if(!AppliNameHere.length())
		AppliNameHere = "NVDEF";
	string UserHere = "";
	if(UserName != NULL)
		UserHere = UserName;
	CleanString(UserHere);
	if(!UserHere.length())
		UserHere = "nvdef";

	int iSessionTimeOut = -1;

	string OpenProcHere = "";
	if(OpenProc != NULL)
		OpenProcHere = OpenProc;
	Session->m_sOpenProc = OpenProcHere;

	string CloseProcHere = "";
	if(CloseProc != NULL)
		CloseProcHere = CloseProc;
	Session->m_sCloseProc = CloseProcHere;

	m_sProcName = "";
	if(ProcName != NULL)
		m_sProcName = ProcName;

	m_sProcNameFailed = "";
	if(ProcNameFailed != NULL)
		m_sProcNameFailed = ProcNameFailed;

	
	Session->m_bFromTransaction = true;

	string PasswordHere ="";

	GetMain()->AttachRequestList.Get(Session, ATTACH_REQUEST_TYPE_THREAD, AppliNameHere);
	int LoginRetVal = Session->Login(UserHere.c_str(), PasswordHere.c_str(), AppliNameHere.c_str(), SESSION_TYPE_THREAD, NULL, NULL, false, false, iSessionTimeOut, NULL, Language);

	if(LoginRetVal != 1)
	{
		// We free the session
		GetMain()->Sessions.FreeSession(Session->m_sIdent.c_str());

		// And remove it
		GetMain()->Sessions.RemoveSession(Session->m_sIdent.c_str());

		return m_sSessionId;
	}
	string UserHereForCheck = UserHere;
	CleanString(UserHereForCheck, true);
	MakeLower(UserHereForCheck);
	if(Session != NULL)
		if(UserHereForCheck == "nvadmin")
			Session->m_bAdministrator = true; // Administrator user
	
	m_sSessionId = Session->m_sIdent;

	if(!Run(false, true))
	{
		// We free the session
		GetMain()->Sessions.FreeSession(Session->m_sIdent.c_str());

		// And remove it
		GetMain()->Sessions.RemoveSession(Session->m_sIdent.c_str());

		m_sSessionId = "";

		return m_sSessionId;
	}

	// We free the session
	GetMain()->Sessions.FreeSession(Session->m_sIdent.c_str());
	
	return m_sSessionId;
}


// called after the Main function (see class NvTask)
void NvSessionThread::OnCleanup()
{
	char sThreadId[50];
	sprintf(sThreadId, "%lu", NvGetCurrentThreadId());
	if(!m_bSameThread)
		GetMain()->Logs.Write("Threads", "Cleanup thread", "Type SESSION", NV_LOG_INFO, 1, sThreadId, "system");

	// We must free the session here

	if(b_Initialized)
	{
		NvSession *Session = NULL;
		if(m_sSessionId.length() != 0)
			Session = GetMain()->Sessions.GetSession(m_sSessionId.c_str());
		if(Session != NULL)
		{
			if(Session->ActualCommand != NULL)
			{
				// The session is in a command so it's probably a crash
				if(Session->ActualCommand->m_sService != "SYSTEM")
					GetMain()->Services.FreeService(Session->ActualCommand->m_sService.c_str());
				Session->ActualCommand = NULL;
				Session->OriginalCommand = NULL;
				Session->InContainer = NULL;
				Session->OutContainer = NULL;
				Session->SetActualCommand("");
				Session->SetOriginalCommand("");
				NV_UNLOCKOBJECT(Session);
			}
			GetMain()->Sessions.FreeSession(Session->m_sIdent.c_str());
			GetMain()->Sessions.RemoveSession(Session->m_sIdent.c_str());
			m_sSessionId = "";
		}

		// Detach thread from java interface if not yet made
		if(!m_bSameThread)
			GetMain()->JavaInterface.DetachJavaThread();
	}

	if(!m_bSameThread)
		GetMain()->Logs.Write("Threads", "Thread stopped", "Type SESSION", NV_LOG_INFO, 1, sThreadId, "system");
	if(!m_bSameThread)
		GetMain()->Threads.Remove();
	NvTask::OnCleanup();	// call the base class
}

